@@ -1,7 +1,3 @@
-{-# LANGUAGE DataKinds          #-}
-{-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE DeriveGeneric      #-}
-{-# LANGUAGE OverloadedStrings  #-}
 -----------------------------------------------------------------------------
 -- |
 -- License     :  BSD-3-Clause
@@ -59,17 +55,12 @@ module GitHub.Endpoints.Repos (
     module GitHub.Data,
     ) where
 
-import Prelude        ()
-import Prelude.Compat
-
-import Control.Applicative ((<|>))
-import Data.Aeson.Compat   (encode)
-import Data.Vector         (Vector)
-
 import GitHub.Data
+import GitHub.Internal.Prelude
 import GitHub.Request
+import Prelude ()
 
-import qualified Data.ByteString.Char8 as BS8
+import qualified Data.Text.Encoding as TE
 
 repoPublicityQueryString :: RepoPublicity -> QueryString
 repoPublicityQueryString RepoPublicityAll     = [("type", Just "all")]
@@ -81,13 +72,13 @@ repoPublicityQueryString RepoPublicityPrivate = [("type", Just "private")]
 -- | List your repositories.
 currentUserRepos :: Auth -> RepoPublicity -> IO (Either Error (Vector Repo))
 currentUserRepos auth publicity =
-    executeRequest auth $ currentUserReposR publicity Nothing
+    executeRequest auth $ currentUserReposR publicity FetchAll
 
 -- | List your repositories.
 -- See <https://developer.github.com/v3/repos/#list-your-repositories>
-currentUserReposR :: RepoPublicity -> Maybe Count -> Request k(Vector Repo)
+currentUserReposR :: RepoPublicity -> FetchCount -> Request k (Vector Repo)
 currentUserReposR publicity =
-    PagedQuery  ["user", "repos"] qs
+    pagedQuery  ["user", "repos"] qs
   where
     qs = repoPublicityQueryString publicity
 
@@ -102,15 +93,19 @@ userRepos = userRepos' Nothing
 -- With authentication.
 --
 -- > userRepos' (Just (BasicAuth (user, password))) "mike-burns" All
-userRepos' :: Maybe Auth -> Name Owner -> RepoPublicity -> IO (Either Error (Vector Repo))
+userRepos'
+    :: Maybe Auth
+    -> Name Owner
+    -> RepoPublicity
+    -> IO (Either Error (Vector Repo))
 userRepos' auth user publicity =
-    executeRequestMaybe auth $ userReposR user publicity Nothing
+    executeRequestMaybe auth $ userReposR user publicity FetchAll
 
 -- | List user repositories.
 -- See <https://developer.github.com/v3/repos/#list-user-repositories>
-userReposR :: Name Owner -> RepoPublicity -> Maybe Count -> Request k(Vector Repo)
+userReposR :: Name Owner -> RepoPublicity -> FetchCount -> Request k(Vector Repo)
 userReposR user publicity =
-    PagedQuery  ["users", toPathPart user, "repos"] qs
+    pagedQuery  ["users", toPathPart user, "repos"] qs
   where
     qs = repoPublicityQueryString publicity
 
@@ -124,15 +119,23 @@ organizationRepos org = organizationRepos' Nothing org RepoPublicityAll
 -- With authentication.
 --
 -- > organizationRepos (Just (BasicAuth (user, password))) "thoughtbot" All
-organizationRepos' :: Maybe Auth -> Name Organization -> RepoPublicity -> IO (Either Error (Vector Repo))
+organizationRepos'
+    :: Maybe Auth
+    -> Name Organization
+    -> RepoPublicity
+    -> IO (Either Error (Vector Repo))
 organizationRepos' auth org publicity =
-    executeRequestMaybe auth $ organizationReposR org publicity Nothing
+    executeRequestMaybe auth $ organizationReposR org publicity FetchAll
 
 -- | List organization repositories.
 -- See <https://developer.github.com/v3/repos/#list-organization-repositories>
-organizationReposR :: Name Organization -> RepoPublicity -> Maybe Count -> Request k (Vector Repo)
+organizationReposR
+    :: Name Organization
+    -> RepoPublicity
+    -> FetchCount
+    -> Request k (Vector Repo)
 organizationReposR org publicity =
-    PagedQuery ["orgs", toPathPart org, "repos"] qs
+    pagedQuery ["orgs", toPathPart org, "repos"] qs
   where
     qs = repoPublicityQueryString publicity
 
@@ -154,7 +157,7 @@ repository' auth user repo =
 -- See <https://developer.github.com/v3/repos/#get>
 repositoryR :: Name Owner -> Name Repo -> Request k Repo
 repositoryR user repo =
-    Query ["repos", toPathPart user, toPathPart repo] []
+    query ["repos", toPathPart user, toPathPart repo] []
 
 -- | Create a new repository.
 --
@@ -165,9 +168,9 @@ createRepo' auth nrepo =
 
 -- | Create a new repository.
 -- See <https://developer.github.com/v3/repos/#create>
-createRepoR :: NewRepo -> Request 'True Repo
+createRepoR :: NewRepo -> Request 'RW Repo
 createRepoR nrepo =
-    Command Post ["user", "repos"] (encode nrepo)
+    command Post ["user", "repos"] (encode nrepo)
 
 -- | Create a new repository for an organization.
 --
@@ -178,27 +181,28 @@ createOrganizationRepo' auth org nrepo =
 
 -- | Create a new repository for an organization.
 -- See <https://developer.github.com/v3/repos/#create>
-createOrganizationRepoR :: Name Organization -> NewRepo -> Request 'True Repo
+createOrganizationRepoR :: Name Organization -> NewRepo -> Request 'RW Repo
 createOrganizationRepoR org nrepo =
-    Command Post ["orgs", toPathPart org, "repos"] (encode nrepo)
+    command Post ["orgs", toPathPart org, "repos"] (encode nrepo)
 
 -- | Edit an existing repository.
 --
 -- > editRepo (BasicAuth (user, password)) "some_user" "some_repo" def {editDescription = Just "some description"}
-editRepo :: Auth
-         -> Name Owner      -- ^ owner
-         -> Name Repo             -- ^ repository name
-         -> EditRepo
-         -> IO (Either Error Repo)
+editRepo
+    :: Auth
+    -> Name Owner      -- ^ owner
+    -> Name Repo             -- ^ repository name
+    -> EditRepo
+    -> IO (Either Error Repo)
 editRepo auth user repo body =
     executeRequest auth $ editRepoR user repo body
 
 
 -- | Edit an existing repository.
 -- See <https://developer.github.com/v3/repos/#edit>
-editRepoR :: Name Owner -> Name Repo -> EditRepo -> Request 'True Repo
+editRepoR :: Name Owner -> Name Repo -> EditRepo -> Request 'RW Repo
 editRepoR user repo body =
-    Command Patch ["repos", toPathPart user, toPathPart repo] (encode b)
+    command Patch ["repos", toPathPart user, toPathPart repo] (encode b)
   where
     -- if no name is given, use curent name
     b = body {editName = editName body <|> Just repo}
@@ -215,17 +219,18 @@ contributors = contributors' Nothing
 -- > contributors' (Just (BasicAuth (user, password))) "thoughtbot" "paperclip"
 contributors' :: Maybe Auth -> Name Owner -> Name Repo -> IO (Either Error (Vector Contributor))
 contributors' auth user repo =
-    executeRequestMaybe auth $ contributorsR user repo False Nothing
+    executeRequestMaybe auth $ contributorsR user repo False FetchAll
 
 -- | List contributors.
 -- See <https://developer.github.com/v3/repos/#list-contributors>
-contributorsR :: Name Owner
-              -> Name Repo
-              -> Bool              -- ^ Include anonymous
-              -> Maybe Count
-              -> Request k (Vector Contributor)
+contributorsR
+    :: Name Owner
+    -> Name Repo
+    -> Bool              -- ^ Include anonymous
+    -> FetchCount
+    -> Request k (Vector Contributor)
 contributorsR user repo anon =
-    PagedQuery ["repos", toPathPart user, toPathPart repo, "contributors"] qs
+    pagedQuery ["repos", toPathPart user, toPathPart repo, "contributors"] qs
   where
     qs | anon      = [("anon", Just "true")]
        | otherwise = []
@@ -246,7 +251,7 @@ contributorsWithAnonymous = contributorsWithAnonymous' Nothing
 -- > contributorsWithAnonymous' (Just (BasicAuth (user, password))) "thoughtbot" "paperclip"
 contributorsWithAnonymous' :: Maybe Auth -> Name Owner -> Name Repo -> IO (Either Error (Vector Contributor))
 contributorsWithAnonymous' auth user repo =
-    executeRequestMaybe auth $ contributorsR user repo True Nothing
+    executeRequestMaybe auth $ contributorsR user repo True FetchAll
 
 -- | The programming languages used in a repo along with the number of
 -- characters written in that language. Takes the repo owner and name.
@@ -268,7 +273,7 @@ languagesFor' auth user repo =
 -- See <https://developer.github.com/v3/repos/#list-languages>
 languagesForR :: Name Owner -> Name Repo -> Request k Languages
 languagesForR user repo =
-    Query  ["repos", toPathPart user, toPathPart repo, "languages"] []
+    query ["repos", toPathPart user, toPathPart repo, "languages"] []
 
 -- | The git tags on a repo, given the repo owner and name.
 --
@@ -282,13 +287,13 @@ tagsFor = tagsFor' Nothing
 -- > tagsFor' (Just (BasicAuth (user, password))) "thoughtbot" "paperclip"
 tagsFor' :: Maybe Auth -> Name Owner -> Name Repo -> IO (Either Error (Vector Tag))
 tagsFor' auth user repo =
-    executeRequestMaybe auth $ tagsForR user repo Nothing
+    executeRequestMaybe auth $ tagsForR user repo FetchAll
 
 -- | List tags.
 -- See <https://developer.github.com/v3/repos/#list-tags>
-tagsForR :: Name Owner -> Name Repo -> Maybe Count -> Request k (Vector Tag)
+tagsForR :: Name Owner -> Name Repo -> FetchCount -> Request k (Vector Tag)
 tagsForR user repo =
-    PagedQuery  ["repos", toPathPart user, toPathPart repo, "tags"] []
+    pagedQuery  ["repos", toPathPart user, toPathPart repo, "tags"] []
 
 -- | The git branches on a repo, given the repo owner and name.
 --
@@ -302,37 +307,38 @@ branchesFor = branchesFor' Nothing
 -- > branchesFor' (Just (BasicAuth (user, password))) "thoughtbot" "paperclip"
 branchesFor' :: Maybe Auth -> Name Owner -> Name Repo -> IO (Either Error (Vector Branch))
 branchesFor' auth user repo =
-    executeRequestMaybe auth $ branchesForR user repo Nothing
+    executeRequestMaybe auth $ branchesForR user repo FetchAll
 
 -- | List branches.
 -- See <https://developer.github.com/v3/repos/#list-branches>
-branchesForR :: Name Owner -> Name Repo -> Maybe Count -> Request k (Vector Branch)
+branchesForR :: Name Owner -> Name Repo -> FetchCount -> Request k (Vector Branch)
 branchesForR user repo =
-    PagedQuery  ["repos", toPathPart user, toPathPart repo, "branches"] []
+    pagedQuery  ["repos", toPathPart user, toPathPart repo, "branches"] []
 
 -- | The contents of a file or directory in a repo, given the repo owner, name, and path to the file
 --
 -- > contentsFor "thoughtbot" "paperclip" "README.md"
-contentsFor :: Name Owner -> Name Repo -> String -> Maybe String -> IO (Either Error Content)
+contentsFor :: Name Owner -> Name Repo -> Text -> Maybe Text -> IO (Either Error Content)
 contentsFor = contentsFor' Nothing
 
 -- | The contents of a file or directory in a repo, given the repo owner, name, and path to the file
 -- With Authentication
 --
 -- > contentsFor' (Just (BasicAuth (user, password))) "thoughtbot" "paperclip" "README.md" Nothing
-contentsFor' :: Maybe Auth ->  Name Owner -> Name Repo -> String -> Maybe String -> IO (Either Error Content)
+contentsFor' :: Maybe Auth ->  Name Owner -> Name Repo -> Text -> Maybe Text -> IO (Either Error Content)
 contentsFor' auth user repo path ref =
     executeRequestMaybe auth $ contentsForR user repo path ref
 
-contentsForR :: Name Owner
-             -> Name Repo
-             -> String            -- ^ file or directory
-             -> Maybe String      -- ^ Git commit
-             -> Request k Content
+contentsForR
+    :: Name Owner
+    -> Name Repo
+    -> Text            -- ^ file or directory
+    -> Maybe Text      -- ^ Git commit
+    -> Request k Content
 contentsForR user repo path ref =
-    Query ["repos", toPathPart user, toPathPart repo, "contents", path] qs
+    query ["repos", toPathPart user, toPathPart repo, "contents", path] qs
   where
-    qs =  maybe [] (\r -> [("ref", Just . BS8.pack $ r)]) ref
+    qs =  maybe [] (\r -> [("ref", Just . TE.encodeUtf8 $ r)]) ref
 
 -- | The contents of a README file in a repo, given the repo owner and name
 --
@@ -350,7 +356,7 @@ readmeFor' auth user repo =
 
 readmeForR :: Name Owner -> Name Repo -> Request k Content
 readmeForR user repo =
-    Query ["repos", toPathPart user, toPathPart repo, "readme"] []
+    query ["repos", toPathPart user, toPathPart repo, "readme"] []
 
 -- | Delete an existing repository.
 --
@@ -359,6 +365,6 @@ deleteRepo :: Auth -> Name Owner -> Name Repo -> IO (Either Error ())
 deleteRepo auth user repo =
     executeRequest auth $ deleteRepoR user repo
 
-deleteRepoR :: Name Owner -> Name Repo -> Request 'True ()
+deleteRepoR :: Name Owner -> Name Repo -> Request 'RW ()
 deleteRepoR user repo =
-    Command Delete ["repos", toPathPart user, toPathPart repo] mempty
+    command Delete ["repos", toPathPart user, toPathPart repo] mempty
